from datetime import datetime, timezone

import pytz
from delta.tables import *

SP = pytz.timezone('America/Sao_Paulo')


def validate_args(args_nedeed: list, args_user: dict):
    """
    Valida se o dicionario passado na funcao consta com os parametros obrigatorios da funcao.

    Args:
        args_nedeed: Lista de configuracoes obrigatorias que a funcao utiliza
        args_user: dicionario de configuracoes do usuario

    Raises:
        KeyError: Caso nao identifique um parametro obrigatorio no dicionario do usuario.

    Examples:
        >>> validate_args(['table_checkpoint_location','table_path'],{'table_checkpoint_location':'/save/_checkpointLocation','table_path':'/save/'})
    """

    for arg in args_nedeed:
        if arg not in args_user:
            raise KeyError(
                f'Nao foi possivel localizar o parametro: {arg} . Por favor adicionar'
            )


def drop_table_path(schema_name: str, table_name: str):

    try:

        spark = SparkSession.builder.getOrCreate()

        if spark.conf.get('spark.databricks.service.client.enabled') == 'true':
            from pyspark.dbutils import DBUtils

            dbutils = DBUtils(spark)
        else:
            import IPython

            dbutils = IPython.get_ipython().user_ns['dbutils']

        table = f'{schema_name}.{table_name}'
        location = (
            DeltaTable.forName(spark, table)
            .detail()
            .select('location')
            .first()[0]
        )

        log_info(f'location da tabela que sera apagada: {location}')

        dbutils.fs.rm(location, True)
        spark.sql(f'DROP TABLE IF EXISTS {table}')

        log_info(f'{table} removida com sucesso')

    except Exception as e:
        log_error(str(e))



from pyspark.sql import SparkSession

from carlton.utils.logger import log_error, log_info


def setup_table(schema_setup_name='bronze'):

    try:

        spark = SparkSession.builder.getOrCreate()

        spark.sql(
            f"""
            CREATE TABLE IF NOT EXISTS {schema_setup_name}.table_config_ingest (
            SCHEMA STRING,
            TABLE STRING,
            DOMAIN STRING,
            REQUESTER STRING
            )
            USING DELTA CLUSTER BY (DOMAIN, SCHEMA)
        """
        )

        log_info(f'{schema_setup_name}.table_config_ingest criado com sucesso')
    except Exception as e:
        log_error(str(e))


def insert_value_table(
    table_name: str,
    domain_name: str,
    requester_name: str,
    schema_name='bronze',
    schema_setup_name='bronze',
):

    try:

        spark = SparkSession.builder.getOrCreate()

        result_query = spark.sql(
            f"SELECT count(1) FROM {schema_setup_name}.table_config_ingest WHERE SCHEMA='{schema_name}' AND TABLE='{table_name}' AND DOMAIN='{domain_name}'"
        ).first()[0]

        if result_query == 1:
            raise Exception(f'{schema_name}.{table_name} ja em uso')

        spark.sql(
            f"INSERT INTO TABLE {schema_setup_name}.table_config_ingest VALUES ('{schema_name}','{table_name}','{domain_name}','{requester_name}')"
        )

        log_info(f'{schema_name}.{table_name} cadastrada com sucesso')
    except Exception as e:
        log_error(str(e))


def get_tables_domain(domain_name: str, schema_setup_name='bronze'):

    try:

        spark = SparkSession.builder.getOrCreate()

        return spark.sql(
            f"SELECT * FROM {schema_setup_name}.table_config_ingest WHERE DOMAIN='{domain_name}'"
        )

    except Exception as e:
        log_error(str(e))


def get_tables_requester(requester: str, schema_setup_name='bronze'):
    try:

        spark = SparkSession.builder.getOrCreate()

        return spark.sql(
            f"SELECT * FROM {schema_setup_name}.table_config_ingest WHERE REQUESTER='{requester}'"
        )

    except Exception as e:
        log_error(str(e))



import logging

# Configurar o logger
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
    handlers=[logging.FileHandler('app.log'), logging.StreamHandler()],
)

# Cria um logger com o nome do módulo atual
logger = logging.getLogger(__name__)


def log_info(message: str):
    """
    Loga uma mensagem de informação.

    Args:
        message (str): A mensagem a ser logada.
    """
    logger.info(message)


def log_warning(message: str):
    """
    Loga uma mensagem de aviso.

    Args:
        message (str): A mensagem a ser logada.
    """
    logger.warning(message)


def log_error(message: str):
    """
    Loga uma mensagem de erro.

    Args:
        message (str): A mensagem a ser logada.
    """
    logger.error(message)


def log_debug(message: str):
    """
    Loga uma mensagem de depuração.

    Args:
        message (str): A mensagem a ser logada.
    """
    logger.debug(message)



import sys

from pyspark.sql import DataFrame, SparkSession
from pyspark.sql.functions import col, current_date

from carlton.ingest.config_ingest import config_ingest_src, config_ingest_tgt
from carlton.ingest.table import read, save
from carlton.mock.eh_send_message import exec
from carlton.utils.helper import validate_args
from carlton.utils.logger import log_error, log_info


def run(args=sys.argv[1:]):
    """
    Função principal para executar a ingestão de dados.

    Args:
        args (list): Lista de argumentos passados para a função.

    Returns:
        DataFrame: DataFrame resultante da operação de ingestão.
    """
    try:

        log_info('Ingestão iniciada')

        # Imprime os argumentos sequencialmente
        for i, arg in enumerate(args):
            print(f'Argument {i}: {arg}')

        # Executa a função de envio de mensagens
        exec(args[0], args[1])
        print('Hello World!')

        # Criação de SparkSession
        # spark = SparkSession.builder.appName(
        #     'Carlton Ingest APP'
        # ).getOrCreate()

        # Exemplo de leitura e salvamento de dados (descomente se necessário)
        # save(
        #     spark,
        #     read(spark, root_properties, custom_config_spark),
        #     root_properties,
        #     custom_config_spark,
        # )

        log_info('Ingestão finalizada')

    except Exception as e:
        # Loga qualquer erro que ocorrer
        log_error(str(e))



import datetime
import json
import os
import random
import time
import uuid

from azure.eventhub import EventData, EventHubProducerClient

from carlton.utils.logger import (  # Certifique-se de importar o log_error e log_info
    log_error,
    log_info,
)


def exec(conn: str, namespace_name: str):
    try:
        EVENT_HUB_CONNECTION_STR = conn
        EVENT_HUB_NAME = namespace_name

        # EVENT_HUB_CONNECTION_STR = "Endpoint=sb://ethstrifedtm.servicebus.windows.net/;SharedAccessKeyName=dtm;SharedAccessKey=rPX394PwwU7nzNNdrh/Q48Sy7UA86YJSF+AEhDSHVvc=;EntityPath=acceptancetesteventhub"
        # EVENT_HUB_NAME = "acceptancetesteventhub"

        # This script simulates the production of events for 10 devices.
        accounts = []
        for x in range(0, 10):
            accounts.append(str(uuid.uuid4()))

        # Create a producer client to produce and publish events to the event hub.
        producer = EventHubProducerClient.from_connection_string(
            conn_str=EVENT_HUB_CONNECTION_STR, eventhub_name=EVENT_HUB_NAME
        )

        for y in range(0, 20):    # For each device, produce 20 events.
            try:
                event_data_batch = (
                    producer.create_batch()
                )   # Create a batch. You will add events to the batch later.
                for acc in accounts:
                    try:
                        # Create a dummy reading.
                        reading = {
                            'account_id': acc,
                            'risk_score': random.randint(0, 100),
                            'risk_level': random.choice(
                                ['low', 'medium', 'high', 'error']
                            ),
                            'assessment_date': '1994-07-27',
                        }
                        s = json.dumps(
                            reading
                        )   # Convert the reading into a JSON string.
                        event_data_batch.add(
                            EventData(s)
                        )   # Add event data to the batch.
                    except Exception as e:
                        log_error(
                            f'Error creating or adding event data: {str(e)}'
                        )
                producer.send_batch(
                    event_data_batch
                )   # Send the batch of events to the event hub.
            except Exception as e:
                log_error(
                    f'Error creating or sending event data batch: {str(e)}'
                )

        # Close the producer.
        producer.close()
    except Exception as e:
        log_error(f'Error in exec function: {str(e)}')



from carlton.utils.helper import validate_args
from carlton.utils.logger import log_error, log_info


def get_params_path(config_ingest: dict) -> dict:

    if config_ingest['file_resource'] == 'adls':

        validate_args(
            [
                'container_src',
                'storage_name_src',
                'container_tgt',
                'storage_name_tgt',
                'path_src',
            ],
            config_ingest,
        )

        config_ingest[
            'carlton_file_path'
        ] = f"abfss://{config_ingest['container_src']}@{config_ingest['storage_name_src']}.dfs.windows.net/{config_ingest['path_src']}/"
        config_ingest[
            'schemaLocation'
        ] = f"abfss://{config_ingest['container_src']}@{config_ingest['storage_name_src']}.dfs.windows.net/{config_ingest['table_name']}/_schemaLocation"
        config_ingest[
            'table_path'
        ] = f"abfss://{config_ingest['container_tgt']}@{config_ingest['storage_name_tgt']}.dfs.windows.net/{config_ingest['table_name']}/"
        config_ingest[
            'checkpointLocation'
        ] = f"abfss://{config_ingest['container_tgt']}@{config_ingest['storage_name_tgt']}.dfs.windows.net/{config_ingest['table_name']}/_checkpointLocation"

    elif config_ingest['file_resource'] == 'dbfs':

        validate_args(
            [
                'file_dbfs',
                'table_name',
            ],
            config_ingest,
        )

        config_ingest[
            'carlton_file_path'
        ] = f"{config_ingest['file_dbfs']}/input/{config_ingest['path_src']}/"
        config_ingest[
            'schemaLocation'
        ] = f"{config_ingest['file_dbfs']}/input/{config_ingest['table_name']}/_schemaLocation"
        config_ingest[
            'table_path'
        ] = f"{config_ingest['file_dbfs']}/output/{config_ingest['table_name']}"
        config_ingest[
            'checkpointLocation'
        ] = f"{config_ingest['file_dbfs']}/output/{config_ingest['table_name']}/_checkpointLocation"

    return config_ingest


def config_ingest_src(config_ingest: dict, custom_config_spark={}) -> dict:
    """
    Prepara as variaveis de configuracao para leitura do arquivo.

    Args:
        custom_config_spark: dicionario de configuracoes adicionais se necessario

    Returns:
        Um dicionario de configuracoes.

    Examples:
        >>> config_ingest_src({'path_src':'/pathfile/', 'file_resource':'dbfs','file_dbfs':'/FileStore/dt_master','table_name':'account_json','type_run':'batch','file_extension':'csv','file_header':'true','file_delimiter':';','table_checkpoint_location':'/save/_checkpointLocation','table_path':'/save/','table_merge_schema':'true'})
        {'pathGlobfilter': '*.csv', 'cloudFiles.format': 'csv', 'cloudFiles.schemaLocation': '/FileStore/dt_master/input/account_json/_schemaLocation', 'cloudFiles.schemaEvolutionMode': 'rescue', 'cloudFiles.inferColumnTypes': 'false', 'cloudFiles.allowOverwrites': 'true', 'rescuedDataColumn': 'carlton_rescued', 'header': 'true', 'delimiter': ';'}
    """

    validate_args(
        [
            'type_run',
            'file_extension',
            'file_resource',
        ],
        config_ingest,
    )

    config_ingest = get_params_path(config_ingest)

    autoloader_config_csv = {}
    if config_ingest['file_extension'] == 'csv':

        validate_args(
            [
                'file_header',
                'file_delimiter',
            ],
            config_ingest,
        )

        autoloader_config_csv = {
            'header': config_ingest['file_header'],
            'delimiter': config_ingest['file_delimiter'],
        }

    autoloader_config_json = {}
    if config_ingest['file_extension'] == 'json':
        autoloader_config_json = {
            'multiLine': True,
            # "encoding": "utf8",
        }

    autoloader_config = {
        'pathGlobfilter': f"*.{config_ingest['file_extension']}",
        'cloudFiles.format': config_ingest['file_extension'],
        'cloudFiles.schemaLocation': config_ingest['schemaLocation'],
        'cloudFiles.schemaEvolutionMode': 'rescue',
        'cloudFiles.inferColumnTypes': 'false',
        'cloudFiles.allowOverwrites': 'true',
        'rescuedDataColumn': 'carlton_rescued',
        **autoloader_config_csv,
        **autoloader_config_json,
        **custom_config_spark,
    }

    return autoloader_config


def config_ingest_tgt(config_ingest: dict, custom_config_spark={}) -> dict:
    """
    Prepara as variaveis de configuracao para leitura do arquivo.

    Args:
        custom_config_spark: dicionario de configuracoes adicionais se necessario

    Returns:
        Um dicionario de configuracoes.

    Examples:
        >>> config_ingest_tgt({'checkpointLocation':'/save/_checkpointLocation','table_path':'/save','table_merge_schema':'true','type_run':'batch','file_extension':'csv','file_header':'true','file_delimiter':';','schema_location':'/input/'})
        {'checkpointLocation': '/save/_checkpointLocation', 'path': '/save', 'mergeSchema': True}
    """

    args_nedeed = [
        'table_path',
        'checkpointLocation',
    ]

    validate_args(args_nedeed, config_ingest)

    save_config = {
        'checkpointLocation': config_ingest['checkpointLocation'],
        'path': config_ingest['table_path'],
        'mergeSchema': True,
        **custom_config_spark,
    }

    return save_config



from pyspark.sql import DataFrame, SparkSession
from pyspark.sql.functions import col, current_date

from carlton.ingest.config_ingest import config_ingest_src, config_ingest_tgt
from carlton.utils.helper import validate_args
from carlton.utils.logger import log_error, log_info


def read(
    config_ingest: dict,
    autoloader_config: dict,
    custom_config_spark={},
) -> DataFrame:
    """
    Lê dados de uma fonte especificada usando as configurações fornecidas.

    Args:
        config_ingest (dict): Configurações de ingestão.
        autoloader_config (dict): Configurações do autoloader.
        custom_config_spark (dict, optional): Configurações personalizadas do Spark.

    Returns:
        DataFrame: DataFrame contendo os dados lidos.
    """
    try:
        # Configura o autoloader com as configurações de ingestão
        autoloader_config = config_ingest_src(
            config_ingest, custom_config_spark
        )

        # Loga as configurações usadas na leitura
        log_info(
            'configuracoes usadas na leitura: ', msg_dict=autoloader_config
        )

        # Cria ou obtém uma sessão Spark
        spark = SparkSession.builder.getOrCreate()

        # Lê os dados da fonte especificada e retorna um DataFrame
        return (
            spark.readStream.format('cloudFiles')
            .options(**autoloader_config)
            .load(config_ingest['carlton_file_path'])
            .select(
                '*',
                current_date().alias('carlton_current_date'),
                col('_metadata').alias('carlton_metadata'),
            )
        )

    except Exception as e:
        # Loga qualquer erro que ocorra durante a leitura
        log_error(str(e))


def save(
    df: DataFrame,
    config_ingest: dict,
    custom_config_spark={},
):
    """
    Salva os dados do DataFrame em um destino especificado usando as configurações fornecidas.

    Args:
        df (DataFrame): DataFrame contendo os dados a serem salvos.
        config_ingest (dict): Configurações de ingestão.
        custom_config_spark (dict, optional): Configurações personalizadas do Spark.
    """
    try:
        # Valida os argumentos necessários
        validate_args(
            ['schema_name', 'table_name', 'table_path', 'type_run'],
            config_ingest,
        )

        # Configura o destino com as configurações de ingestão
        save_config = config_ingest_tgt(config_ingest, custom_config_spark)

        # Loga as configurações usadas na escrita
        log_info('configuracoes usadas na escrita: ', msg_dict=save_config)

        # Cria ou obtém uma sessão Spark
        spark = SparkSession.builder.getOrCreate()

        # Lista de colunas internas que não devem ser incluídas na tabela
        lst_builtin = ['_rescued', 'carlton_current_date', 'carlton_metadata']
        columns_file = ', '.join(
            f'{col.lower()} STRING'
            for col in df.columns
            if col not in lst_builtin
        )

        # Loga informações sobre a tabela que está sendo criada
        log_info(
            f"cadastrando tabela {config_ingest['table_name']} no schema {config_ingest['schema_name']}"
        )
        log_info(
            f'utilizando liquid_cluster. Gerenciando pela coluna carlton_current_date'
        )

        # Cria a tabela no destino especificado
        query_create_table = f"""
            CREATE TABLE IF NOT EXISTS {config_ingest['schema_name']}.{config_ingest['table_name']} (
            {columns_file}
            ,_rescued STRING
            ,carlton_current_date DATE
            ,carlton_metadata struct<file_path:string,file_name:string,file_size:bigint,file_block_start:bigint,file_block_length:bigint,file_modification_time:timestamp>
            )
            USING DELTA CLUSTER BY (carlton_current_date) LOCATION '{config_ingest['table_path']}'"""

        log_info(f'cadastrando tabela: {query_create_table}')
        spark.sql(query_create_table)
        log_info(
            f"tabela {config_ingest['schema_name']}.{config_ingest['table_name']} criada com sucesso"
        )

        # Configura o tipo de trigger para a gravação dos dados
        type_trigger = {}
        if config_ingest['type_run'] == 'batch':
            log_info(f'Identificando execucao como batch')
            type_trigger['availableNow'] = True
        else:
            log_info(f'Identificando execucao como stream')
            validate_args(['trigger_processing_time'], config_ingest)
            type_trigger['processingTime'] = config_ingest[
                'trigger_processing_time'
            ]

        log_info(f'iniciando gravacao dos registros')

        # Grava os dados no destino especificado
        df.writeStream.options(**save_config).outputMode('append').trigger(
            **type_trigger
        ).table(
            f"{config_ingest['schema_name']}.{config_ingest['table_name']}"
        ).awaitTermination()

        log_info(
            f"{config_ingest['schema_name']}.{config_ingest['table_name']} processada com sucesso"
        )

    except Exception as e:
        # Loga qualquer erro que ocorra durante a gravação
        log_error(str(e))
